/*

Copyright (C) 2016 Freshdesk, Inc.

This source code is a part of the Freshdesk SDK and is covered by the our license
terms. For details about this license, please read the LICENSE.txt which is
bundled with this source code.

*/
'use strict';

const debuglog = __debug.bind(null, __filename);

var _ = require('underscore');
var fileUtil = require('../utils/file-util');
var util = require('../utils/helper');
var eh = require('../utils/err');

var appClassifier = require('../utils/app-classifier');

module.exports = {
  run: function(validationType) {
    var appType = appClassifier();

    var validationPath = __dirname + '/../validations/';
    var validationFiles = fileUtil.readDir(validationPath);
    var errMessage = [];

    for (var i in validationFiles) {
      var validator = require(validationPath + validationFiles[i]);
      if (_.contains(validator.validationType, validationType)) {
        debuglog(`Running ${validationFiles[i]}`);

        var err = validator.validate(appType);
        // Check for promise and add to return
        if (err) {
          errMessage.push(err);
        }
      }
    }

    return _.compact(_.flatten(errMessage));
  }
};
