'use strict';

const debuglog = __debug.bind(null, __filename);

var request = require('request');
var pjson = require('../../package.json');

/*
  Function to compare two versions and return if its the latest.
*/
function isGreaterThan(oldVersion, newVersion) {
  var oldV = oldVersion.split(".");
  var oMajor = Number(oldV[0]);
  var oMinor = Number(oldV[1]);
  var oPatch = Number(oldV[2]);

  var newV = newVersion.split(".");
  var nMajor = Number(newV[0]);
  var nMinor = Number(newV[1]);
  var nPatch = Number(newV[2]);

  if (nMajor > oMajor) {
    return true;
  }
  else if (nMajor === oMajor) {
    if (nMinor > oMinor) {
      return true;
    }
    else if (nMinor === oMinor) {
      if (nPatch > oPatch) {
        return true;
      }
    }
  }
  return false;
}

module.exports = {
  run: function() {
    console.log(`Installed: ${pjson.version}` )

    //Perform API Call to the version management to get the latest version.
    request({
      url: "http://ws.freshdev.io/cli/version.json",
      method: "GET"
    }, function(err, req, body) {
      if (err) {
        debuglog(`Error while checking for version ${err.message}`);

        if (err.code === "ENOTFOUND") {
          console.log("Unable to contact remote server to fetch latest version. Please make sure you are connected to internet.");
          process.exit(1);
        }
      }
      try {
        var versionData = JSON.parse(body);
        if (pjson.version === versionData.fdkCli.version) {
          console.log('Already Up to Date..!')
        }
        if (isGreaterThan(pjson.version, versionData.fdkCli.version)) {
          console.log(
            `Latest: ${versionData.fdkCli.version}\n` +
            `To install the latest version type '${versionData.fdkCli.cmd}'`
          )
        }
      }
      catch (e) {
        debuglog(`Error while checking for version ${e.message}`);
      }
    })
  }
}
