var jQueryDeferred = require('jquery-deferred');
var request = require("request");
var _ = require("underscore");

var SUCCESS_STATUS = [200, 201];
var MISSING_ATTR_ERR = {"message": "Mandatory attributes (key or value) is missing", "status": 400};
var DPROUTER_URL = "http://localhost:10001/dprouter";
var CONTENT_TYPE = "application/json";
var DB_FEATURE = "db";
var POST = "post";

var makeCall = function(body) {
  var dbDeferred = jQueryDeferred.Deferred();
  if (body.action === "store" && !body.data) {
    return dbDeferred.reject(MISSING_ATTR_ERR);
  }
  if (!body.dbKey) {
    return dbDeferred.reject(MISSING_ATTR_ERR);
  }
  request({
    method: POST,
    url: DPROUTER_URL,
    headers: {
      "MKP-ROUTE": DB_FEATURE,
      "Content-Type": CONTENT_TYPE
    },
    json: true,
    body: body
  }, function(error, response, body) {
    if (_.contains(SUCCESS_STATUS, response.statusCode)) {
      dbDeferred.resolve(body);
    }
    else {
      dbDeferred.reject(body);
    }
  });
  return dbDeferred;
}

module.exports = {
  set: function(key, val) {
    return makeCall({dbKey: key, data: val, action: "store"});
  },
  get: function(key) {
    return makeCall({dbKey: key, action: "fetch"});
  },
  delete: function(key) {
    return makeCall({dbKey: key, action: "delete"});
  }
}
