var jQueryDeferred = require('jquery-deferred');
var _ = require('underscore');
var uuid = require('uuid');

const EVENT_NAME = 'onExternalEvent';
const MISSING_EVENT_CALLBACK_ERR = "Unsupported operation: The generateTargetUrl method cannot be used unless there is a corresponding external event callback";
const UNSUPPORTED_OPERATION = "Unsupported operation: The generateTargetUrl method cannot be used";
const UNSUPPORTED_OPTION = "Invalid argument type: The 'options' argument should be of type string or cannot be greater than 100 characters";
const MAX_LENGTH = 100;
var EXTERNAL_EVENT_SUPPORT = ['onTicketCreate', 'onTicketUpdate', 'onConversationCreate',
                    'onContactCreate', 'onContactUpdate', 'onAppInstall',
                    'onExternalEvent'];

module.exports = function(self, options) {
  var genHookDefer = jQueryDeferred.Deferred();
  var eventsArray = _.pluck((self.svrExecScript.events || []), "event");
  var isNotRegistered = !_.contains(eventsArray, EVENT_NAME);
  if (isNotRegistered) {
    genHookDefer.reject({ message: MISSING_EVENT_CALLBACK_ERR });
  }
  else if (!_.contains(EXTERNAL_EVENT_SUPPORT, self.event.categoryArgs.methodName)) {
    genHookDefer.reject({ message: UNSUPPORTED_OPERATION });
  }
  else if (options && (!_.isString(options) || options.trim("") === "" || options.length > MAX_LENGTH)) {
    genHookDefer.reject({ message: UNSUPPORTED_OPTION });
  }
  else {
    var source = `http://localhost:10001/${uuid.v4()}`;
    var url = options ? `${source}?options=${encodeURIComponent(options)}` : source;
    genHookDefer.resolve(url);
  }
  return genHookDefer;
}
