/*

Copyright (C) 2016 Freshdesk, Inc.

This source code is a part of the Freshdesk SDK and is covered by the our license
terms. For details about this license, please read the LICENSE.txt which is
bundled with this source code.

*/
'use strict';

const debuglog = __debug.bind(null, __filename);

var fs   = require('fs');
var fileUtil = require('./utils/file-util');
var _ = require('underscore');
var process = require("process");
var manifestFile = './manifest.json';
var serverFile = './server/server.js';
var oauthFile = './config/oauth_config.json';

var inferFeatures = function() {
  // Always support `db`...
  var inferredFeatures = [ 'db' ];

  if (fs.existsSync(serverFile)) {
    inferredFeatures.push('backend');
  }

  if (fs.existsSync(oauthFile)) {
    inferredFeatures.push('oauth');
  }

  debuglog(`Inferred ${inferredFeatures} as features.`);

  return inferredFeatures;
};

/*
  reload function to update the manifest details to the module.
  usage: require("manifest").reload() will update the details.
*/

var reload = function () {
  if (!fs.existsSync(manifestFile)) {
    console.log('Could not find the app manifest file in the current/specified directory.')
    return process.exit(1);
  }
  try {
    var doc = JSON.parse(fs.readFileSync(manifestFile, 'utf8'));
  }
  catch (e) {
    console.log('Could not parse the app manifest file in the current/specified directory.')
    return process.exit(1);
  }

  debuglog(`Reloading manifest with ${JSON.stringify(doc)}.`);

  module.exports.manifest = doc;
  // Easy access to common use properties:
  module.exports.pfVersion = doc['platform-version'];
  module.exports.features = inferFeatures();
  module.exports.product = doc['product'];
  module.exports.whitelistedDomains = doc['whitelisted-domains'] || [];
  module.exports.dependencies = doc['dependencies'] || {};
}

module.exports = {
  reload: reload,
  manifest: null,
  pfVersion: null,
  product: null,
  features: null,
  whitelistedDomains: null,
  dependencies: null
};

// Load the manifest the first time:
reload();
