'use strict';

const debuglog = __debug.bind(null, __filename);

var _ = require('underscore');
var eventHandler = require('../event_handler/framework');
var configUtil = require('../utils/config-util');
var fileUtil = require('../utils/file-util');
var httpUtil = require("../utils/http-util");
var manifest = require('../manifest');
var eventUtil = require('../utils/event-util');

const CATEGORY_NAME = 'productEvent';

const WAIT_FOR_RESPONSE_EVENTS = ['onAppInstall', 'onAppUninstall'];

const BAD_REQUEST_STATUS = 400;

const GATEWAY_TIMED_OUT = 504;

function iParamConfigs() {
  const iparams = configUtil.getValuesForLocalTesting();

  debuglog(`Using ${JSON.stringify(iparams)} as iparams for testing.`);

  return iparams;
}

function handleRequest(req, res) {
  var product = Object.keys(manifest.product)[0];

  if (!eventUtil.isValidEvent(product, req.query.name)) {
    var errorMsg = `Invalid event: ${req.query.name}`;
    console.log(errorMsg);
    res.status(BAD_REQUEST_STATUS).send({ message: errorMsg });
    return;
  }

  var methodParams = JSON.parse(fileUtil.readFile(`${process.cwd()}/server/test_data/${req.query.name}.json`));
  methodParams.iparams = iParamConfigs();
  methodParams.event = req.query.name;

  if (!_.isEmpty(req.body)) {
    methodParams.data = req.body;
  }
  
  var options = {
    categoryName: CATEGORY_NAME,
    categoryArgs: {
      methodName: req.query.name,
      methodParams: methodParams
    }
  };
  req.body = options;

  debuglog(`Calling event handler with ${JSON.stringify(req.body)}`);

  eventHandler(req, res);

  if(_.contains(WAIT_FOR_RESPONSE_EVENTS, req.query.name)) {
    setTimeout(function() {
      if(!res.headersSent) {
        console.log('`renderData` was not called in the callback within time limit!');
        res.status(GATEWAY_TIMED_OUT).send({ message: 'Request Timed Out!' });
      }
    }, 10000);
  }
  else {
    res.send();
  }
}

module.exports = {
  run: function run(app) {
    app.use(function use(req, res, next) {
      httpUtil.enableCORS(res);
      next();
    });
    app.post('/event/execute', handleRequest);
  }
};
