/*

Copyright (C) 2016 Freshdesk, Inc.

This source code is a part of the Freshdesk SDK and is covered by the our license
terms. For details about this license, please read the LICENSE.txt which is
bundled with this source code.

*/
'use strict';

const debuglog = __debug.bind(null, __filename);

var configUtil = require("../utils/config-util");
var manifest = require("../manifest");
var oauth = require('../routes/oauth2');
var path = require('path');
var _ = require('underscore');

function constructV2Response(req) {
  const currentProduct = req.query.product;

  if (!manifest.product[currentProduct]) {
    return [];
  }

  const locations = JSON.parse(JSON.stringify(manifest.product[currentProduct].location));

  function convertURLs(locations, oauth) {
    _.forEach(locations, (value, key) => {
      value.url = 'http://localhost:10001/iframe/' + (oauth || value.url);
    });
  }

  const appDetails = {
    id: 1,
    version_id: 1,
    configs: configUtil.getValuesForLocalTesting(),
    displayName: process.cwd().split(path.sep).pop(),
    placeholders: locations
  };

  if (_.contains(manifest.features, 'oauth') && !_.has(oauth.fetchCredentials(), 'access_token')) {
     convertURLs(appDetails.placeholders, 'oauth/local-testing-oauth-msg.html');
  } else {
    convertURLs(appDetails.placeholders);
  }

  return [ appDetails ];
}

var iframeAPIHandlerV2 = function(req, res) {
  const appDetails = constructV2Response(req);

  debuglog(`Responding to product with ${JSON.stringify(appDetails)}`);

  res.send(appDetails);
}

var iframeAPIHandler = function(req, res) {
  var appDetails = {
    configs: configUtil.getValuesForLocalTesting(),
    displayName: process.cwd().split(path.sep).pop(),
    product: manifest.product,
    features: manifest.features
  };
  if (_.contains(manifest.features, 'oauth')) {
    var isAuthorized = false;
    if (_.has(oauth.fetchCredentials(), 'access_token')) {
      isAuthorized = true;
    }
    _.extend(appDetails, { isAuthorized: isAuthorized });
  }

  appDetails.v2 = constructV2Response(req);

  debuglog(`Responding to product with ${JSON.stringify(appDetails)}`);

  res.send(appDetails);
}

module.exports = {
  run: function(app) {
    app.get('/iframe/api', iframeAPIHandler);
    app.get('/iframe/api/v2', iframeAPIHandlerV2);
  }
};
