/*

Copyright (C) 2016 Freshdesk, Inc.

This source code is a part of the Freshdesk SDK and is covered by the our license
terms. For details about this license, please read the LICENSE.txt which is
bundled with this source code.

*/
'use strict';

const debuglog = __debug.bind(null, __filename);

var fileUtil = require('./file-util');
var manifest = require('../manifest');
var _ = require('underscore');
var appType = [];

function isPureBackend() {
  var appFolderPath = process.cwd() + '/app/';

  /*
    If the app is not a backend app, it can't be a pure backend app.
  */
  if (!_.contains(manifest.features, 'backend')) {
    return;
  }

  /*
    Even if one of the product contains location, it can't be a pure backend app.
  */
  if (Object.keys(manifest.product || {}).some(key => 'location' in manifest.product[key])) {
    return;
  }

  /*
    If the app has an app directory with files in it, it can't be a pure backend app.
  */
  if (fileUtil.fileExists(appFolderPath) && fileUtil.readDir(appFolderPath).length !== 0) {
    return;
  }

  return appType.push('purebackend');
}

module.exports = function() {
  [
    isPureBackend
  ].forEach(e => e());

  return appType;
};