/*

Copyright (C) 2016 Freshdesk, Inc.

This source code is a part of the Freshdesk SDK and is covered by the our license
terms. For details about this license, please read the LICENSE.txt which is
bundled with this source code.

*/

'use strict';

const debuglog = __debug.bind(null, __filename);

var _ = require('underscore');
var fileUtil = require('./file-util');
var eh = require("./err");

var testDataFile = 'iparam_test_data.json';
var jsonFile = 'iparams.json';
var htmlFile = 'iparams.html';
var oauthConfigFile = 'oauth_config.json';
var nsUtil = require('./ns-resolver');
var storage = require('./data-store');

var HTML_FILE = `${process.cwd()}/config/iparams.html`;
var JSON_FILE = `${process.cwd()}/config/iparams.json`;

function getJSONContent(fileName) {
  var data = fileUtil.readFile(process.cwd() + `/config/${fileName}`);
  try {
    return JSON.parse(data);
  }
  catch (e) {
    console.log(`${fileName}:`);
    eh.error(new Error(e));
  }
}

module.exports = {
  getConfig: function getConfig() {
    return getJSONContent(jsonFile);
  },

  getValuesForLocalTesting: function getValuesForLocalTesting() {
    if (fileUtil.fileExists(HTML_FILE)) {
      return storage.fetch(nsUtil.getInternalNamespace('custom_iparams'));
    }
    else if (fileUtil.fileExists(JSON_FILE)) {
      return getJSONContent(testDataFile);
    }
  },

  getHTMLContent() {
    return fileUtil.readFile(process.cwd() + `/config/${htmlFile}`);
  },

  getOAuthIparam() {
    return getJSONContent(oauthConfigFile)
  }
};
