'use strict';

const debuglog = __debug.bind(null, __filename);

var fs = require('fs');

const fdkFolder = `${process.cwd()}/.fdk/`;
const localStoreFile = `${fdkFolder}localstore`;

var readLocalStore = function() {
  var jsonData;

  try {
    if (!fs.existsSync(fdkFolder)) {
      fs.mkdirSync(fdkFolder);
    }
    jsonData = JSON.parse(fs.readFileSync(localStoreFile));
    debuglog(`Read ${JSON.stringify(jsonData)}`);
  }
  catch (e) {
    debuglog(`Error while reading localstore ${e.message}`);
    jsonData = {};
  }

  return jsonData;
}

module.exports = {
  store: function(key, data) {
    var jsonData = readLocalStore();
    jsonData[key] = data;
    fs.writeFileSync(localStoreFile, JSON.stringify(jsonData));
    return {"Created": true};
  },

  fetch: function(key) {
    return readLocalStore()[key];
  },

  delete: function(key) {
    var jsonData = readLocalStore();
    delete jsonData[key];
    fs.writeFileSync(localStoreFile, JSON.stringify(jsonData));
    return {"Deleted": true};
  }
};
