/*

Copyright (C) 2016 Freshdesk, Inc.

This source code is a part of the Freshdesk SDK and is covered by the our license
terms. For details about this license, please read the LICENSE.txt which is
bundled with this source code.

*/
'use strict';

const debuglog = __debug.bind(null, __filename);

var crypto = require('crypto');
var eh = require('./err');
var fileUtil = require('./file-util');
var fs = require('fs');
var nodeDir = require('node-dir');

var digestErrorMsg = "Error while generating digest file.";
var ENCODING = 'binary';

function writeToDigest(files, callback) {
  files.push('manifest.json');
  debuglog(`Computing digest for ${files}.`);

  var content = '';

  try {
    for (var file of files) {
      content += fileUtil.readFile(`${process.cwd()}/${file}`, ENCODING);
    }
    var digest = crypto.createHash('md5').update(content, ENCODING).digest("hex");
    fileUtil.writeFile(`${process.cwd()}/digest.md5`, digest);
    callback();
  } catch(e) {
    eh.error(new Error(digestErrorMsg));
  }
}

module.exports = {

  genDigestFile: function(callback) {
    var files = [];
    if (fs.existsSync('./app')) {
      fileUtil.getFiles({dir: './app'}, function(err, appFiles) {
        if (err) {
          eh.error(new Error(digestErrorMsg));
        }
        files = files.concat(appFiles);
        writeToDigest(files, callback);
      });
    } else {
      writeToDigest(files, callback);
    }
  },

  delDigestFile: function() {
    debuglog('Deleting digest.');
    fileUtil.deleteFile(`${process.cwd()}/digest.md5`);
  }
};
