'use strict';

const debuglog = __debug.bind(null, __filename);

var manifest = require('../manifest');
var manifestValidation = require('../validations/manifest-dependency-validation');
var npm = require('npm');
var _ = require('underscore');
var nsUtil = require('../utils/ns-resolver');
var storage = require('../utils/data-store');
var eh = require('../utils/err');
var crypto = require('crypto');

// Used to check if dependencies has modification
var dpnHash = crypto.createHash('md5').update(JSON.stringify(manifest.dependencies)).digest('hex');

function installDependency(dependencies, callback) {
  var installPath = `${process.cwd()}/server`;
  var dpnArr = [];
  npm.load({ loaded: false, loglevel: "silent", depth: 0, progress: false, prefix: installPath}, function (error) {
    if (error) { return callback(error.message); };
    //check for the versions before installing.
    npm.commands.ls(Object.keys(dependencies), true, function(err, data, list) {
      if (err) { return callback(err.message); }
      var installedDependencies = list.dependencies || {};
      for(var dpn in dependencies) {
        if (!installedDependencies[dpn] || (dependencies[dpn] != installedDependencies[dpn].version) ) {
          dpnArr.push(`${dpn}@${dependencies[dpn]}`);
        }
      }
      if(dpnArr.length == 0) {
        debuglog('No dependencies to be installed.');
        return callback(null); 
      }
      debuglog(`Preparing to install ${dpnArr}`);
      console.log('Installing dependencies...');
      npm.commands.install(installPath, dpnArr, function (er) {
        if (er) { return callback(er.message); };
        console.log("### Installation succeeded ###");
        callback(null);
      });
    });
  });
}

function validateDependencies(callback) {
  var dependencyErr = manifestValidation.validateDependencies();
  storage.store(nsUtil.getInternalNamespace('dpn_hash'), dpnHash);
  if (!(_.isEmpty(dependencyErr))) {
    return callback(dependencyErr);
  }
}

module.exports = {
  run: function(callback) {
    /*
      Run the validation only if the dependencies are modified
    */
    if (storage.fetch(nsUtil.getInternalNamespace('dpn_hash')) !== dpnHash) {
      validateDependencies(callback);
    }
    var dependencies = manifest.dependencies;
    if (_.isEmpty(dependencies)) {
      console.log("### No dependencies to install ###");
      return callback(null);
    }
    installDependency(dependencies, callback);
  }
};
