/*

Copyright (C) 2016 Freshdesk, Inc.

This source code is a part of the Freshdesk SDK and is covered by the our license
terms. For details about this license, please read the LICENSE.txt which is
bundled with this source code.

*/
'use strict';

const debuglog = __debug.bind(null, __filename);

var path = require('path');

var PACKAGE_EXTENSION = ".zip"

function rootFolderName() {
  return process.cwd().split(path.sep).pop();
}

function normalize(folderName) {
  var MAX_CHARS = 6;
  var processed = folderName.toLowerCase().replace(/[^a-zA-Z0-9]/g, "_");
  if (processed.length > MAX_CHARS) {
    processed = processed.substring(0, MAX_CHARS);
  }
  return processed + "_101_101";
}

function getNamespace() {
  const folderName = rootFolderName();

  return { "app_id" : normalize(folderName)};
}

var internalNameSpace = {
  /*
    Namespace to store custom Iparams data in file
  */
  custom_iparams: function() {
    return `${getNamespace()['app_id']}_custom_iparams`;
  },

  /*
    Namespace to store md5 hash of dependencies of the project
  */
  dpn_hash: function() {
    return `${getNamespace()['app_id']}_dpn_hash`;
  },

  /*
    Namespace to store schedule events
  */
  schedule: function(meta) {
    return `${getNamespace()['app_id']}_schedule_${meta}`;
  }
}

module.exports = {

  getNamespace,

  getRootFolder: function () {
    return rootFolderName();
  },

  getInternalNamespace: function(key, meta) {
    return internalNameSpace[key](meta);
  },

  pkgExt: PACKAGE_EXTENSION
};
