/*

Copyright (C) 2016 Freshdesk, Inc.

This source code is a part of the Freshdesk SDK and is covered by the our license
terms. For details about this license, please read the LICENSE.txt which is
bundled with this source code.

*/
'use strict';

const debuglog = __debug.bind(null, __filename);

var validationConst = require('./constants').validationContants;
var fileUtil = require('../utils/file-util');
var _ = require('underscore');

var MAX_INDVL_ASSET_SIZE = 2000000; // size in bytes. 2000000 bytes = 2 MB.

module.exports = {
  validate: function(appType) {
    var errMsgs = []
    var appFolderPath = process.cwd() + '/app/';
    if (_.contains(appType, 'purebackend')) {
      return;
    }
    if (fileUtil.fileExists(appFolderPath)) {
      var appFiles = fileUtil.readDir(appFolderPath);
      appFiles = appFiles.filter(fileUtil.removeJunkFiles);


      for (var i in appFiles) {
        var filePath = appFolderPath + appFiles[i];
        var fileStat = fileUtil.statFile(filePath);

        //Asset size check
        if (fileStat.size > MAX_INDVL_ASSET_SIZE) {
          errMsgs.push("App file exceeds threshold size(2MB): app/" + appFiles[i]);
        }
      }
    }
    else {
      errMsgs.push("Mandatory folder(s) missing: app")
    }
    return errMsgs;
  },

  validationType: [validationConst.PRE_PKG_VALIDATION, validationConst.RUN_VALIDATION]
};
