/*
Copyright (C) 2016 Freshdesk, Inc.
This source code is a part of the Freshdesk SDK and is covered by the our license
terms. For details about this license, please read the LICENSE.txt which is
bundled with this source code.
*/
'use strict';

const debuglog = __debug.bind(null, __filename);

var validationConst = require('./constants').validationContants;
var manifest = require('../manifest');
var _ = require('underscore');
var syncRequest = require('sync-request');

function validateDependencies() {
  var err = [];
  for (var i in manifest.dependencies) {
    var registryResponse = syncRequest('GET', `http://registry.npmjs.org/${i}`);
    if (registryResponse.statusCode === 404) {
       err.push(`Unknown dependency mentioned in manifest.json: ${i}.`);
    }
    else {
      var availableVersions = _.keys(JSON.parse(registryResponse.body).versions);
      var version = String(manifest.dependencies[i]);
      if (version.startsWith('^') || version.startsWith('~')) {
        err.push(`Package '${i}' version contains special characters(^ or ~).`);
        version = version.slice(1);
      }
      if (!(_.contains(availableVersions, version))) {
        err.push(`Package ${i} does not have version ${manifest.dependencies[i]}.`);
      }
      else {
        var downloadCountRequest = syncRequest('GET', `https://api.npmjs.org/downloads/point/last-month/${i}`);
        var downloadCount = JSON.parse(downloadCountRequest.body.toString()).downloads;
        if (downloadCount < 50) {
          err.push(`Package ${i} does not reach the minimum popularity.`)
        }
      }
    }
  }
  return err;
}

module.exports = {
  validate: function() {
    var errMsgs = [];
    var dependencyErr = validateDependencies();
    if (!(_.isUndefined(dependencyErr))) {
      errMsgs.push(dependencyErr);
    }
    return _.flatten(errMsgs);
  },
  validateDependencies : validateDependencies,
  validationType: [validationConst.PRE_PKG_VALIDATION]
};
