/*

Copyright (C) 2016 Freshdesk, Inc.

This source code is a part of the Freshdesk SDK and is covered by the our license
terms. For details about this license, please read the LICENSE.txt which is
bundled with this source code.

*/

'use strict';

const debuglog = __debug.bind(null, __filename);

var fs = require('fs-extra');
var manifest = require('./manifest');
var fileUtil = require('./utils/file-util');

var files = ['./app', './config', './manifest.json'];

module.exports = {
  watch: function(broadcast) {
    var filesToWatch = [];
    files.forEach(function(file) {
      if (fileUtil.fileExists(`${process.cwd()}/${file}`)) {
        filesToWatch.push(file);
      }
    });

    debuglog(`Preparing to watch the following files ${filesToWatch}`);

    filesToWatch.forEach(function(fileToWatch) {
      fs.watch(fileToWatch, {persistent: false, recursive: true}, function(event, file) {

        // Reload cache:
        if (file === 'manifest.json') {
          manifest.reload();
        }

        // Broadcast:
        var broadcastMessage = JSON.stringify({ file });

        broadcast(broadcastMessage);
        debuglog(`Broadcasted "${broadcastMessage}" to all listeners.`);
      });
    });
  }
};
