/*

Copyright (C) 2016 Freshdesk, Inc.

This source code is a part of the Freshdesk SDK and is covered by the our license
terms. For details about this license, please read the LICENSE.txt which is
bundled with this source code.

*/

var configUtil = require('../../utils/config-util');
var manifest = require('../../manifest');
var oauth = require('../../routes/oauth2');
var fileUtil = require('../../utils/file-util');
var nsUtil = require('../../utils/ns-resolver');
var storage = require('../../utils/data-store');
var _ = require('underscore');

/*
  Save custom Iparams data to storage (file) with namespace
*/
var saveIParamsToDB = function(data) {
  storage.store(nsUtil.getInternalNamespace('custom_iparams'), data);
}

/*
  Fetch custom IParams data from storage (file) with namespace
*/
var getIParamsFromDB = function() {
  return storage.fetch(nsUtil.getInternalNamespace('custom_iparams'));
}

/*
  Route handler - fetch
  Get custom IParams from storage
  If exists return data else return {}
*/
var fetchCustomIParams = function (req, res) {
  var customIParams = getIParamsFromDB();
  if (_.isNull(customIParams) || _.isUndefined(customIParams)) {
    customIParams = {};
  }
  return res.send({
    isCustomIParam: fileUtil.fileExists(`${process.cwd()}/config/iparams.html`),
    customIParams
  });
}

/*
  Route handler - To store custom IParams
  Store posted custom IParams to DB
*/
var storeCustomIParams = function (req, res) {
  saveIParamsToDB(req.body);
  res.send();
}

/*
  Route handler - To fetch custom installation Page
  If app is oauth & not authorized
    -> Authorize and redirect to form page
*/
var customIParamsPage = function(req, res) {
  if (_.contains(manifest.features, 'oauth') && !_.has(oauth.fetchCredentials(), 'access_token')) {
    res.writeHead(302, {
      'Location': 'http://localhost:10001/auth/index?callback=http://localhost:10001/custom_configs'
    });
    return res.end();
  }
  return res.send(fileUtil.readFile(__dirname + '/../pages/custom-iparams.html'));
}

/*
  Route handler - To fetch custom installation page form content
*/
var customIParamsForm = function(req, res) {
  if (fileUtil.fileExists(`${process.cwd()}/config/iparams.html`)) {
    return res.send(fileUtil.readFile(`${process.cwd()}/config/iparams.html`));
  }
  return res.send();
}

module.exports = {
  run: function(app) {
    app.get('/custom_configs', customIParamsPage);
    app.get('/custom_configs/form', customIParamsForm);
    app.get('/custom_configs/fetch', fetchCustomIParams);
    app.post('/custom_configs/store', storeCustomIParams);
  }
}
