var fileUtil = require('../../utils/file-util');
var manifest = require('../../manifest');
var eventUtil = require('../../utils/event-util');
var _ = require('underscore');

var getEventData = function(req, res) {
  var event = req.params.event;
  try {
    var eventData = JSON.parse(fileUtil.readFile(`${process.cwd()}/server/test_data/${event}.json`));
    res.json(eventData);
  }
  catch (err) {
    console.log(`Error while parsing ${event}.json`)
    res.json({});
  }
}

var storeEventData = function(req, res) {
  var event = req.params.event;
  fileUtil.writeFile(`${process.cwd()}/server/test_data/${event}.json`, JSON.stringify(req.body, null, 2));
  res.send();
}

var resetEventData = function(req, res) {
  var event = req.params.event;
  var product = Object.keys(manifest.product)[0];
  var eventData = require(`../../../config/payloads/${product}/${event}.json`);
  fileUtil.writeFile(`${process.cwd()}/server/test_data/${event}.json`, JSON.stringify(eventData, null, 2));
  res.json(eventData);
}

var eventsPage = function(req, res) {
  if (_.contains(manifest.features, 'backend')) {
    res.send(fileUtil.readFile(__dirname + '/../pages/event-page.html'));
  }
  else {
    res.send(fileUtil.readFile(__dirname + '/../pages/event-page-404.html'));
  }
}

var eventsList = function(req, res) {
  var product = Object.keys(manifest.product)[0];
  res.send({ events: eventUtil.eventsList(product) });
}

module.exports = {
  run: function(app) {
    app.get('/web/events', eventsPage);
    app.get('/web/eventsList', eventsList);
    app.get('/web/events/:event', getEventData);
    app.post('/web/events/:event', storeEventData);
    app.post('/web/events/reset/:event', resetEventData);
  }
}
