var tmp = require('tmp');
var expect = require('chai').expect;
var fs = require('fs-extra');
var configDirValidator;

describe('config dir validate', function() {

  var projDir;

  before(function() {
    projDir = tmp.dirSync({prefix: 'freshapps_sdk'});
    require(__dirname + '/../lib/cli/init').run(projDir['name'], 'your_first_app');
    process.chdir(projDir['name']);
    require(__dirname + '/../lib/manifest').reload();
    configDirValidator = require('../lib/validations/config-dir-validation')
  });

  it('should validate empty config dir', function(done) {
    expect([]).eql(configDirValidator.validate());
    done();
  });

  it('should not fail for valid files', function(done) {
    var validIparamFile = __dirname + '/../test-res/iparam/valid_iparam.json';
    var iparamJSONFile = projDir['name'] + '/config/iparams.json';
    fs.copySync(validIparamFile, iparamJSONFile);
    expect([]).eql(configDirValidator.validate());

    var validIparamTestFile = __dirname + '/../test-res/iparam/valid_iparam_test_data.json';
    var iparamJSONTestFile = projDir['name'] + '/config/iparam_test_data.json';
    fs.copySync(validIparamTestFile, iparamJSONTestFile);
    expect([]).eql(configDirValidator.validate());
    
    var validOauthFile = __dirname + '/../test-res/iparam/valid_iparam.json';
    var oauthJSONFile = projDir['name'] + '/config/oauth_config.json';
    fs.copySync(validOauthFile, oauthJSONFile);
    expect([]).eql(configDirValidator.validate());

    var validHtmlFile = __dirname + '/../test-res/iparam/iparams.html';
    var htmlFile = projDir['name'] + '/config/iparams.html';
    fs.copySync(validHtmlFile, htmlFile);
    expect([]).eql(configDirValidator.validate());
    done();
  })

  it('should fail when other than json/html available', function(done) {
    var validIparamFile = __dirname + '/../test-res/iparam/valid_iparam.json';
    var destIparamFile = projDir['name'] + '/config/invalid_iparam_file_type.txt';
    fs.copySync(validIparamFile, destIparamFile);
    expect(["Config directory has invalid file(s) - invalid_iparam_file_type.txt"]).eql(configDirValidator.validate());
    done();
  });

});
