var tmp = require('tmp');
var request = require('supertest');
var pjson = require('../package.json');
var expect = require('chai').expect;
global.pjson = pjson;

describe('Persistence test', function () {
  var projDir, server;

  var db = require(__dirname + '/../lib/event_handler/features/db');
  before( function() {
    projDir = tmp.dirSync({prefix: 'freshapps_sdk'});
    require(__dirname + '/../lib/cli/init').run(projDir['name'], 'your_first_app');
    process.chdir(projDir['name']);
    require(__dirname + '/../lib/manifest').reload();
  });

  beforeEach(function () {
    server = require('../lib/cli/run').run();
  });

  afterEach(function () {
    server.close();
  });

  it('should store', function testSlash(done) {
    var payload = {
      'data' : {
        'info' : 'hello world'
      },
      'dbKey' : 'user1',
      'action' : 'store'
    }
    request(server)
      .post('/dprouter')
      .send(payload)
      .set({
        "content-type":"application/json",
        "MKP-EXTNID": "101",
        "MKP-VERSIONID": "101",
        "MKP-ROUTE": "db"
      })
      .expect(201,done)
  })

  it('should not fail store - 201', function testSlash(done) {
    var payload = {
      'data' : {
        "one": []
      },
      'dbKey' : 'user1',
      'action' : 'store'
    }
    request(server)
      .post('/dprouter')
      .send(payload)
      .set({
        "content-type":"application/json",
        "MKP-EXTNID": "101",
        "MKP-VERSIONID": "101",
        "MKP-ROUTE": "db"
      })
      .expect(201,done)
  })

  it('should not fail store - 201', function testSlash(done) {
    var payload = {
      'data' : {
        "one": ''
      },
      'dbKey' : 'user1',
      'action' : 'store'
    }
    request(server)
      .post('/dprouter')
      .send(payload)
      .set({
        "content-type":"application/json",
        "MKP-EXTNID": "101",
        "MKP-VERSIONID": "101",
        "MKP-ROUTE": "db"
      })
      .expect(201,done)
  })

  it('should fail store - blank data - 400', function testSlash(done) {
    var payload = {
      'dbKey' : 'user1',
      'action' : 'store'
    }
    request(server)
      .post('/dprouter')
      .send(payload)
      .set({
        "content-type":"application/json",
        "MKP-EXTNID": "101",
        "MKP-VERSIONID": "101",
        "MKP-ROUTE": "db"
      })
      .expect(400,done)
  })

  it('should fail store - data is greater than 4096', function testSlash(done) {
    var payload = {
      'data' : {
        'hello': new Array(8196).fill('1').join('')
      },
      'dbKey' : 'user1',
      'action' : 'store'
    }
    request(server)
      .post('/dprouter')
      .send(payload)
      .set({
        "content-type":"application/json",
        "MKP-EXTNID": "101",
        "MKP-VERSIONID": "101",
        "MKP-ROUTE": "db"
      })
      .expect(400,done)
  })

  it('should fetch', function testSlash(done) {
    setTimeout(function() {
      var payload = {
        'dbKey' : 'user1',
        'action' : 'fetch'
      }
      request(server)
        .post('/dprouter')
        .send(payload)
        .set({
          "content-type":"application/json",
          "MKP-EXTNID": "101",
          "MKP-VERSIONID": "101",
          "MKP-ROUTE": "db"
        })
       .expect(200,done)
    }, 1000)
  })

  it('should fail fetch - 404', function testSlash(done) {
    setTimeout(function() {
      var payload = {
        'dbKey' : 'user11311111111',
        'action' : 'fetch'
      }
      request(server)
        .post('/dprouter')
        .send(payload)
        .set({
          "content-type":"application/json",
          "MKP-EXTNID": "101",
          "MKP-VERSIONID": "101",
          "MKP-ROUTE": "db"
        })
       .expect(404,done)
    }, 1000)
  })

  it('should delete', function testSlash(done) {
    var payload = {
      'dbKey' : 'user2',
      'action' : 'delete'
    }
    request(server)
      .post('/dprouter')
      .send(payload)
      .set({
        "content-type":"application/json",
        "MKP-EXTNID": "101",
        "MKP-VERSIONID": "101",
        "MKP-ROUTE": "db"
      })
      .expect(200,done)
  })

  it('should not fail - missing key', function testSlash(done) {
    var payload = {
      'data' : {
        'info' : 'hello world'
      },
      'dbKey' : '',
      'action' : 'store'
    }
    request(server)
      .post('/dprouter')
      .send(payload)
      .set({
        "content-type":"application/json",
        "MKP-EXTNID": "101",
        "MKP-VERSIONID": "101",
        "MKP-ROUTE": "db"
      })
      .expect(400,done)
  })

  it('should fail - key length > 30', function testSlash(done) {
    var payload = {
      'data' : {
        'info' : 'hello world'
      },
      'dbKey' : 'abcdefghijklmnopqrstuvwxyz12345678900987',
      'action' : 'store'
    }
    request(server)
      .post('/dprouter')
      .send(payload)
      .set({
        "content-type":"application/json",
        "MKP-EXTNID": "101",
        "MKP-VERSIONID": "101",
        "MKP-ROUTE": "db"
      })
      .expect(400,done)
  })

  it('should fail - key within data length > 30', function testSlash(done) {
    var payload = {
      'data' : {
        'abcdefghijklmnopqrstuvwxyz12345678900987abcdefghijklmnopqrstuvwxyz12345678900987npm' : 'hello world'
      },
      'dbKey' : 'hello',
      'action' : 'store'
    }
    request(server)
      .post('/dprouter')
      .send(payload)
      .set({
        "content-type":"application/json",
        "MKP-EXTNID": "101",
        "MKP-VERSIONID": "101",
        "MKP-ROUTE": "db"
      })
      .expect(400,done)
  })

  it('should not fail - 11 keys', function testSlash(done) {
    var payload = {
      'data' : {
        'info1' : 'hello world',
        'info2' : 'hello world',
        'info3' : 'hello world',
        'info4' : 'hello world',
        'info5' : 'hello world',
        'info6' : 'hello world',
        'info7' : 'hello world',
        'info8' : 'hello world',
        'info9' : 'hello world',
        'info10' : 'hello world',
        'info11' : 'hello world'
      },
      'dbKey' : 'user3',
      'action' : 'store'
    }
    request(server)
      .post('/dprouter')
      .send(payload)
      .set({
        "content-type":"application/json",
        "MKP-EXTNID": "101",
        "MKP-VERSIONID": "101",
        "MKP-ROUTE": "db"
      })
      .expect(201,done)
  })

  it('should fail - data must be json', function testSlash(done) {
    var payload = {
      'data' : "hello world",
      'dbKey' : 'user3',
      'action' : 'store'
    }
    request(server)
      .post('/dprouter')
      .send(payload)
      .set({
        "content-type":"application/json",
        "MKP-EXTNID": "101",
        "MKP-VERSIONID": "101",
        "MKP-ROUTE": "db"
      })
      .expect(422,done)
  })


  it('backend - should store', function testSlash(done) {
    db.set('user1', {'info': 'hello world'})
    .then(function(data) {
      expect(data).eql({Created:true});
      done()
    })
  })

  it('backend - should fetch', function testSlash(done) {
    setTimeout(function() {
      db.get('user1')
      .then(function(data) {
        expect(data).eql({info: 'hello world'});
        done()
      })
    }, 1000);
  })

  it('should fail store - 201', function testSlash(done) {
    var payload = {
      'data' : {
        "one": {
          '': 'asas'
        }
      },
      'dbKey' : 'user1',
      'action' : 'store'
    }
    request(server)
      .post('/dprouter')
      .send(payload)
      .set({
        "content-type":"application/json",
        "MKP-EXTNID": "101",
        "MKP-VERSIONID": "101",
        "MKP-ROUTE": "db"
      })
      .expect(400,done)
  })

  it('backend - should delete', function testSlash(done) {
    db.delete('user1')
    .then(function(data) {
      expect(data).eql({Deleted: true});
      done()
    })
  })

  it('backend - should fail storing - missing attributes', function testSlash(done) {
    db.set('user1')
    .then(function(data) {
    })
    .fail(function(err) {
      expect(err).eql({message: "Mandatory attributes (key or value) is missing", status: 400})
      done();
    })
  })

  it('backend - should replace empty strings and NaN with null', function testSlash(done) {
    db.set('user1', {
      hello: '',
      world: ['asas', ''],
      fai: {
        foo: NaN
      }
    })
    .then(function(data) {
      db.get('user1')
        .then(function(data) {
          expect(data).to.deep.equal({
            hello: null,
            world: ['asas', null],
            fai: {
              foo: null
            }
          });
          done();
        })
        .fail(done);
    })
    .fail(done)
  })

  it('backend - should fail storing - missing attributes', function testSlash(done) {
    db.set(undefined, {"one": "true"})
    .then(function(data) {
    })
    .fail(function(err) {
      expect(err).eql({message: "Mandatory attributes (key or value) is missing", status: 400})
      done();
    })
  })

  it('backend - should fail storing - data must be json', function testSlash(done) {
    db.set('user1', 3)
    .then(function(data) {
    })
    .fail(function(err) {
      expect(err).eql({message: "The value cannot be blank and should be of type JSON", status: 422})
      done();
    })
  })
});
