var tmp = require('tmp');
var request = require('supertest');
var fs = require('fs-extra');
var expect = require('chai').expect;

describe('Backend app file scope test', function () {
  var projDir, server;

  before( function() {
    projDir = tmp.dirSync({prefix: 'freshapps_sdk'});
    require(__dirname + '/../lib/cli/init').run(projDir['name'], 'your_first_serverless_app');
    process.chdir(projDir['name']);

    var testServerFile = __dirname + '/../test-res/file_scope_server.js';
    var testLibFile = __dirname + '/../test-res/file_scope_lib.js';
    var serverFile = projDir['name'] + '/server/server.js';
    var libFile = projDir['name'] + '/server/lib/handle-response.js';
    fs.copySync(testServerFile, serverFile);
    fs.copySync(testLibFile, libFile);

    require(__dirname + '/../lib/manifest').reload();
    delete require.cache[require.resolve(__dirname + '/../lib/event_handler/framework')];
    server = require('../lib/cli/run').run();
  });

  after(function () {
    server.close();
  });

  it ('should fail when local variable in lib file used in server file', function testSlash(done) {
    var payload = {
      methodName: 'callRemote',
      methodParams: { url: 'https://www.google.co.in/#q=freshdesk', localVarTest: true },
      action: 'invoke'
    };
    var headers = {
      'content-type': 'application/json',
      'MKP-EXTNID': '101',
      'MKP-VERSIONID': '101',
      'MKP-ROUTE': 'smi'
    };
    request(server)
     .post('/dprouter')
     .send(payload)
     .set(headers)
     .expect(200, function(error, resp) {
        var response = resp.body;
        expect(response.status).to.equal(500);
        expect(response.message).eql('Error while executing App server script! - localVar is not defined');
        done();
     });
  });

  it ('should pass when global variable in lib file used in server file', function testSlash(done) {
    var payload = {
      methodName: 'callRemote',
      methodParams: { url: 'https://www.google.co.in/#q=freshdesk', globalVarTest: true },
      action: 'invoke'
    };
    var headers = {
      'content-type': 'application/json',
      'MKP-EXTNID': '101',
      'MKP-VERSIONID': '101',
      'MKP-ROUTE': 'smi'
    };
    request(server)
     .post('/dprouter')
     .send(payload)
     .set(headers)
     .expect(200, function(error, resp) {
        var response = resp.body;
        expect(Object.keys(response)).to.deep.equal(['requestID']);
        done();
     });
  });
});
