var tmp = require('tmp');
var fs = require('fs-extra');
var expect = require('chai').expect;

describe('test install', function() {
  var projDir, dependencyInstaller, manifest;

  before(function() {
    projDir = tmp.dirSync({prefix: 'freshapps_sdk'});
    require(__dirname + '/../lib/cli/init').run(projDir['name'], 'your_first_serverless_app');
    process.chdir(projDir['name']);
    dependencyInstaller = require(`${__dirname}/../lib/utils/install`);
    manifest = require(`${__dirname}/../lib/manifest`);
  });

  it ('should successfully return when no dependencies present', function(done) {
    manifest.reload();
    dependencyInstaller.run(function(err) {
      expect(err).eql(null);
      done();
    });
  });

  it ('should successfully install the dependencies when present', function(done) {
    var destManifestFile = `${projDir['name']}/manifest.json`;
    var srcManifestFile = `${__dirname}/../test-res/manifest/manifest_valid_dependency.json`;
    fs.copySync(srcManifestFile, destManifestFile);
    manifest.reload();  
    dependencyInstaller.run(function(err) {
      var serverDirs = fs.readdirSync(`${projDir['name']}/server`);
      expect(serverDirs).to.include('node_modules');
      var modules = fs.readdirSync(`${projDir['name']}/server/node_modules`);
      expect(modules).to.include('fs-extra');
      done();
    });
  });

  it ('should install if a new npm dependency is added', function(done) {
    var destManifestFile = `${projDir['name']}/manifest.json`;
    var srcManifestFile = `${__dirname}/../test-res/manifest/manifest_new_dependency.json`;
    fs.copySync(srcManifestFile, destManifestFile);
    manifest.reload();
    dependencyInstaller.run(function(err) {
      var modules = fs.readdirSync(`${projDir['name']}/server/node_modules`);
      expect(modules).to.include('request');
      done();
    });
  });

});