var tmp = require('tmp');
var request = require('supertest');
var pjson = require('../package.json');
var fs = require('fs');
var expect = require('chai').expect;
var WebSocket = require('websocket').w3cwebsocket;
global.pjson = pjson;

describe('Cli run test', function () {
  var projDir, server;

  before(function() {
    projDir = tmp.dirSync({prefix: 'freshapps_sdk'});
    require(__dirname + '/../lib/cli/init').run(projDir['name'], 'your_first_app');
    process.chdir(projDir['name']);
  });

  beforeEach(function () {
    server = require('../lib/cli/run').run();
  });

  afterEach(function () {
    server.close();
  });

  it('expects 200 for iframe api', function testSlash(done) {
    request(server)
      .get('/iframe/api')
      .expect(200,done)
  });

  it('expects 200 for iframe api v2', function testSlash(done) {
    request(server)
      .get('/iframe/api/v2')
      .expect(200, done)
  });

  it('check if app content changes are pushed through websocket', function testSlash(done) {
    var ws = new WebSocket("ws://localhost:10001/notify-change");
    ws.onmessage = function(e){
      expect(e.data).to.eql('{"file":"template.html"}');
      ws.close();
      done();
    };
    fs.writeFileSync(`${projDir.name}/app/template.html`, 'dummy data');
  });

  it('check if manifest is reloaded on change', function testSlash(done) {
    var mf = require('../lib/manifest');
    var modifiedManifest = JSON.parse(fs.readFileSync(`${projDir.name}/manifest.json`));
    modifiedManifest['whitelisted-domains'] = ['https://www.google.co.in'];
    var ws = new WebSocket("ws://localhost:10001/notify-change");
    ws.onmessage = function(e){
      expect(e.data).to.eql('{"file":"manifest.json"}');
      expect(mf.whitelistedDomains).to.eql([ 'https://www.google.co.in' ]);
      ws.close();
      done();
    };
    setTimeout(function() {
      fs.writeFileSync(`${projDir.name}/manifest.json`, JSON.stringify(modifiedManifest));
    }, 1000);
  });

});
