var expect = require('chai').expect;
var tmp = require('tmp');
var request = require('supertest');
var fs = require('fs-extra');
var nodeFs = require('fs');

describe('Web event test', function () {
  var projDir, server;

  before( function() {
    projDir = tmp.dirSync({prefix: 'freshapps_sdk'});
    require(__dirname + '/../lib/cli/init').run(projDir['name'], 'your_first_serverless_app');
    process.chdir(projDir['name']);
    var testServerFile = __dirname + '/../test-res/backend_server.js';
    var serverFile = projDir['name'] + '/server/server.js';
    require(__dirname + '/../lib/manifest').reload();
    fs.copySync(testServerFile, serverFile);
    delete require.cache;
    server = require('../lib/cli/run').run();
  });

  after(function () {
    server.close();
  });

  it('should get Event Data - onTicketCreate', function testSlash(done) {
    request(server)
      .get('/web/events/onTicketCreate')
      .send()
      .expect(200,done)
  })

  it('should get Event Data - onTicketCreate', function testSlash(done) {
    nodeFs.writeFileSync(projDir['name'] + '/server/test_data/onContactCreate.json', "hello")
    request(server)
      .get('/web/events/onContactCreate')
      .send()
      .expect(200,done)
  })

  it('should get Event page - onTicketCreate', function testSlash(done) {
    request(server)
      .get('/web/events')
      .send()
      .expect(200,done)
  })

  it('should store Event page - onTicketCreate', function testSlash(done) {
    request(server)
      .post('/web/events/onTicketCreate')
      .send({})
      .expect(200,done)
  })

  it('should reset Event Data - onTicketCreate', function testSlash(done) {
    request(server)
      .post('/web/events/reset/onTicketCreate')
      .send()
      .expect(200,done)
  })

  it('should get 404 event page', function testSlash(done) {
    fs.removeSync(projDir['name'] + '/server/server.js')
    require(__dirname + '/../lib/manifest').reload();
    request(server)
      .get('/web/events')
      .send()
      .expect(200,done)
  })

  it('should fetch list of events for a product', function(done) {
    request(server)
      .get('/web/eventsList')
      .send()
      .expect(200, function(err, resp){
        expect(resp.body.events.length).eql(9);
        done();
      })
  });
});
